﻿using System;
using System.Data.SqlClient;
using FreD.Data;
using FreD.Data.Providers;
using FreD.Improve.Prediction;

namespace MyPredictor
{
	/// <summary>
	/// siehe hierzu: 
	/// J. Michael: Anredebestimmung anhand des Vornamens; erschienen in: c't 17/07; Heise Verlag; Hannover; 2007; S. 182-
	/// und http://www.heise.de/ct/ftp/07/17/182/
	/// </summary>
	/// 
	[Serializable()]
	public class GenderPredictor : BasePredictor
	{
		protected override void DoBuild( string table, string fields, string fieldClass, string where )
		{
			if( fields.ToLower() != "name" )
				throw new ArgumentException( "Fields must be 'name'" );

			if( fieldClass == null || ( fieldClass.ToLower() != "gender" && fieldClass.ToLower() != "country" ) )
				throw new ArgumentException( "Field class must be 'Gender' or 'Country'" );
		}

		protected override object DoPredict( FreD.Data.Row row )
		{
			using( SqlServerProvider provider = new SqlServerProvider() )
			{
				string name = ( ( string )row.Values[ 0 ] ).Replace( ' ', '+' );

				using( IReader reader = provider.GetTableReader( 
										"SELECT * FROM q.Lookup.Gender WHERE Name = @name", 
										new SqlParameter[] { new SqlParameter( "@name", name ) } ) )
				{
					row = reader.Read( 0 );

					if( row != null )
					{
						if( FieldClass.ToLower() == "gender" )
							return translateGender( ( ( string )row[ 0 ] ).Trim() );

						return translateCountry( row, reader.Columns );
					}
				}
			}

			return null;
		}


		private string translateGender( string gender )
		{
			if( gender == "M" )
				return "male";
			else if( gender == "1M" )
				return "male, if first part of name";
			else if( gender == "?M" )
				return "male, mostly";
			else if( gender == "F" )
				return "female";
			else if( gender == "1F" )
				return "female, if first part of name";
			else if( gender == "?F" )
				return "female, mostly";
			else if( gender == "?" )
				return "any gender";

			return null;
		}

		private string translateCountry( FreD.Data.Row row, FreD.Data.Columns columns )
		{
			char max = ' ';
			string column = null;

			// ab 2.tem Attribut beginnen die Länderhäufigkeiten
			for( int i = 2; i < row.Values.Length; i++ )
			{
				string count = ( string )row.Values[ i ];

				if( count[ 0 ] > max )
				{
					max = count[ 0 ];
					column = columns[ i ].Name;
				}
			}
			return column;
		}
	}
}
