using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using Microsoft.SqlServer.Server;

using FreD.Understand.Properties;

public class SchemaPropertiesBySQL : ISchemaPropertyList
{
	public List<SchemaItem> Process( string table, string column )
	{
		List<SchemaItem> properties = new List<SchemaItem>();

		using( SqlConnection con = new SqlConnection( SqlContext.IsAvailable ?
									"context connection=true" :
									"Data Source=(local);Initial Catalog=AdventureWorks;Integrated Security=True" ) )
		{
			// database und schema aus Tabellennamen ermitteln
			string database = null;
			string schema = null;

			string[] items = table.Split( new char[] { '.' }, StringSplitOptions.RemoveEmptyEntries );

			if( items.Length < 3 )
				throw new ArgumentException( "SchemaPropertiesBySQL expected the table variable in the form: database.schema.table" );

			database = items[ 0 ];
			schema = items[ 1 ];
			table = items[ 2 ];
			
			con.Open();

			string sql = string.Format( "SELECT * " +
										"FROM INFORMATION_SCHEMA.COLUMNS " +
										"WHERE TABLE_CATALOG = '{0}' AND " +
										"TABLE_SCHEMA = '{1}' AND " +
										"TABLE_NAME = '{2}'", 
										database, schema, table);
			
			if( column.Trim() != "*" )
			{
				string[] columns = column.Split( new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries );
				
				column = "'";
				for(int i=0; i<columns.Length; i++ )
					column += columns[ i ].Trim() + ( i == columns.Length-1 ? "'" : "', '" );
			}

			sql += string.Format( " AND COLUMN_NAME IN ({0})", column );

			SqlCommand cmd = new SqlCommand( sql );
			cmd.Connection = con;

			using( SqlDataReader r = cmd.ExecuteReader() )
			{
				while( r.Read() )
				{
					object[] values = new object[ r.FieldCount ];

					r.GetValues( values );

					for( int i = 0; i < values.Length; i++ )
					{
						string propertyName = r.GetName( i );
						string value = values[ i ] != null ? values[ i ].ToString() : "NULL";
						properties.Add( new SchemaItem( r[ "COLUMN_NAME" ].ToString(), propertyName, value, "" ) );
					}
				}
			}
		}
		
		return properties;
	}
}